#!/usr/bin/perl

use strict;

##########################################################################
#  Remove Attachments script.
#   2004/03 Niversoft ides logicielles
#  Script by Nicolas Hatier
##########################################################################
#
#  Copy this script in a subdirectory of the Communigate pro base folder,
#  for instance /var/Communigate/Filters (create the subfolder if it
#  doesn't exists).
#
#  In the accounts you don't want to receive attachments, add this rule:
#
#	Header Field  is not X-Attachments-Removed: yes
#
#  Action: Execute   [RETPATH][RCPT] /usr/bin/perl /var/Communigate/Filters/RemoveAttachments.pl
#          Discard
#
#  The filter will remove attachments if any and resubmit the mail via the
#  PIPE module. The X-Attachments-Removed: header wil prevent mail loops.
#  WARNING! This WILL broke any digital signature received with the message since
#  the message will have been modified.
#
#  Nicolas Hatier, nicolas dot hatier at niversoft dot com
#


# You can get these packages via your preferred CPAN archive if you don't have them on your system
use MIME::Tools;
use MIME::Parser;
use MIME::Decoder::QuotedPrint;
use MIME::Decoder::Base64;
use MIME::Decoder::Binary;
use MIME::Decoder::Gzip64;
use MIME::Decoder::NBit;
use MIME::Decoder::UU;
use MIME::Words qw(:all);


my $rundir = $0;
$rundir =~ s|\\|/|g;
$rundir = index($rundir, "/")>=0 ? substr($rundir, 0, rindex($rundir, "/")) : ".";

exit(main());

sub ReplaceAttachements
{
	my @newparts = ();

	foreach my $part ($_[0]->parts)
	{
	   print ((scalar $part->parts) . " " . $part->mime_type() . "\n");

	   if (scalar $part->parts)
	   {
	      ReplaceAttachements($part);
	      push @newparts, $part;
	   }
	   else
	   {
         if ($part->mime_type() =~ m,^text/|^plain/,i)
   		{
            push @newparts, $part;
   		}
   		else
   		{
   		   my $filename = $part->head->recommended_filename();
   		   push @newparts, MIME::Entity->build(
   		      Data => "Attachment named '$filename' has been removed from this message.",
   		      Type =>"text/plain",
   		      Encoding => "quoted-printable"
   		   );
   		}
   	}
	}
	$_[0]->parts(\@newparts);
}

sub main()
{
	my $parser = new MIME::Parser;
	$parser->output_to_core(0);
	$parser->extract_nested_messages(0);
	$parser->tmp_to_core(1);
	$parser->tmp_recycling(1);
	$parser->use_inner_files(0);
	$parser->filer->ignore_filename(1);

	my @message = <STDIN>;
	my $entity = $parser->parse_data(\@message);

	ReplaceAttachements($entity);

	my $dest = 0;
	my @recipients;
	my $returnpath;
	foreach (@ARGV)
	{
	   if(/^-p$/)
	   {
	      $dest = 0;
	   }
	   elsif(/^-r$/)
	   {
	      $dest = 1;
	   }
	   elsif($dest)
	   {
	      push @recipients, $_;
	   }
	   else
	   {
	      $returnpath = $_;
	   }
	}

	my $filename = "Submitted/outfile".time;
	$filename .= "_" while (-f $filename);

	open OUTPUT, ">$filename.tmp";
	print OUTPUT "Return-Path: $returnpath\n";
	foreach (@recipients)
	{
	   print OUTPUT "Envelope-To: $_\n";
	}
	print OUTPUT "X-Attachments-Removed: yes\n";

	$entity->print(\*OUTPUT);
	close OUTPUT;
	rename "$filename.tmp", "$filename.sub";
	$parser->filer->purge();

	return 0;
}

